﻿
//-------------------------------------------
// The GREP search matches things like
// 'See also UXP; See also JavaScript; See also AppleScript'
// The space after the xref string is a variable-width non-breaking space,
// it's always inserted by InDesign.
// Xrefs are sorted, spurious see alsos removed, xrefs moved down

(function () {

	var xrefString = 'See also';
	var moveCrossRefs = true;
	
	var parts;
	var indd = app.documents[0];
	var betweenSep = indd.indexGenerationOptions.betweenEntriesSeparator;
	var xrefStyle = indd.indexGenerationOptions.crossReferenceStyle;
	var beforeXrefSep = indd.indexGenerationOptions.beforeCrossReferenceSeparator;

	var par;
	var nextPar;
	var nextParName;
	var lastPar;

	function nextParagraph (par) {
		return par.characters[-1].contents === '\r' ? par.insertionPoints[-1].paragraphs[0] : null;
	}

	function previousParagraph (par) {
		return par.index !== 0 ? par.parentStory.insertionPoints[par.index-1].paragraphs[0] : null;
	}

	// Find the index. If there's a selection and the selected story
	// is an index, return the story. If there's no selection, 
	// try and find an index story.

	function findIndex () {
		if (app.selection.length 
				&& app.selection[0].hasOwnProperty('parentStory') 
				&& app.selection[0].parentStory.storyType === StoryTypes.INDEXING_STORY) {
			return app.selection[0].parentStory;
		}
		if (app.documents[0].indexes.length > 0) {
			var stories = app.documents[0].stories.everyItem().getElements();
			for (var i = stories.length-1; i >= 0; i--) {
				if (stories[i].textContainers[0].parent instanceof Spread 
						&& stories[i].storyType === StoryTypes.INDEXING_STORY) {
					return stories[i];
				}
			}
		}
		return null;
	}	

	var story = findIndex();
	if (!story) {
		alert ('No generated index in this document');
		exit();
	}

	// Delete any story-trailing space
	
	while (story.characters[-1].contents === '\r') {
		story.characters[-1].contents = '';
	}

	//==============================================================
	
	// Find all strings from the xref string to the end of the paragraph

	app.findGrepPreferences = null;
	app.findGrepPreferences.findWhat = xrefString + '\\s.+'
	var xrefs = story.findGrep();
	// Create a regex like /See also\s/g
	var re = RegExp (xrefString + '\\s', 'g');
	for (var i = xrefs.length-1; i >= 0; i--) {
		// Delete the xref strings and split on the separator
		parts = xrefs[i].contents.replace (re,'').split(betweenSep);
		// Only one xref: no need to continue
		if (parts.length > 1) {
			// Because InDesign is what it is, we have to get rid
			// of all instances of the xref character style
			xrefs[i].appliedCharacterStyle = app.documents[0].characterStyles[0];
			xrefs[i].clearOverrides (OverrideType.CHARACTER_ONLY);
			// Insert the xref string followed by the sorted xref topics
			xrefs[i].contents = xrefString + ' ' + parts.sort().join(betweenSep);
			// Apply the xref style to the xref string
			xrefs[i].characters.itemByRange (0, xrefString.length-1).appliedCharacterStyle = xrefStyle;
		}
		
		//----------------------------------------------
		if (!moveCrossRefs) {
			continue;
		}
		// Move the xrefs
		// Find the parent topic's last topic with page reference
		nextPar = nextParagraph (xrefs[i].paragraphs[0]);
		nextParName = nextPar.appliedParagraphStyle.name;
		while (nextPar != null && nextPar.appliedParagraphStyle.name === nextParName) {
			nextPar = nextParagraph (nextPar);
		}

		if (nextPar == null) { // lastPar is the story's last par
			lastPar = story.paragraphs[-1];
		} else {
			
			lastPar = previousParagraph (nextPar);
		}
	
		par = xrefs[i].paragraphs[0];
		// Add an empty paragraph after the last subtopic
		lastPar.insertionPoints[-1].contents = '\r';
		// Move the xrefs
		xrefs[i].move (LocationOptions.AFTER, lastPar.insertionPoints[-1]);
		// InDesign quirk. We added a return after lastPar,
		// which now therefore consists of two paragraphs
		// We have to target the last one.
		lastPar.paragraphs[-1].applyParagraphStyle (lastPar.appliedParagraphStyle, false);
		
		// Remove the before-xref-separator after the parent topic's name
		story.recompose();
		par.contents = par.contents.replace (RegExp (beforeXrefSep+'$'), '');	

	}
}());
